import 'package:flutter/material.dart';
import 'package:flutter_slick/navigation/item.dart';
import 'package:flutter_slick/navigation/logic.dart';
import 'package:flutter_slick/navigation/route_path.dart';
import 'package:flutter_slick/utils/color_extensions.dart';
import 'package:provider/provider.dart';

class StandardMenuItem<User> extends StatelessWidget {
  final IconData? iconData;
  final String titleText;
  final RoutePath? routePath;
  final List<NavigationItem>? children;
  final BoxDecoration? decoration;
  final BoxDecoration? selectedDecoration;

  const StandardMenuItem({
    Key? key,
    required this.titleText,
    required this.routePath,
    this.iconData,
    this.children,
    this.decoration,
    this.selectedDecoration,
  })  : assert(
            routePath == null && children != null ||
                routePath != null && children == null,
            'routePath or onTap needs to be set but not both.'),
        super(key: key);

  @override
  Widget build(BuildContext context) {
    var path = Provider.of<NavigationLogic>(context, listen: false).routePath;

    return Column(
      children: [
        if (routePath == null)
          const Divider(
            height: 1,
          ),
        _buildMenuItem(context, routePath, iconData, titleText, path),
        if (children != null)
          ...children!.map(
            (child) => _buildMenuItem(
              context,
              child.routePath,
              child.iconData,
              child.labelText,
              path,
            ),
          ),
      ],
    );
  }

  Widget _menuItem(
    BuildContext context,
    IconData? iconData,
    String labelText,
    bool isSelected,
  ) {
    return Padding(
      padding: EdgeInsets.only(
        left: 6,
        right: 6,
        top: Theme.of(context).visualDensity == VisualDensity.compact ? 6 : 12,
        bottom:
            Theme.of(context).visualDensity == VisualDensity.compact ? 6 : 12,
      ),
      child: Container(
        padding:
            const EdgeInsets.only(left: 8.0, right: 8.0, top: 10, bottom: 10),
        decoration: _decoration(context, isSelected),
        child: Row(
          children: [
            const SizedBox(width: 4),
            if (iconData != null) Icon(iconData),
            if (iconData != null) const SizedBox(width: 4),
            Text(labelText),
          ],
        ),
      ),
    );
  }

  BoxDecoration? _decoration(BuildContext context, bool isSelected) {
    if (!isSelected) {
      if (decoration != null) {
        return decoration!;
      } else {
        return null;
      }
    } else {
      if (selectedDecoration != null) {
        return selectedDecoration!;
      } else {
        return BoxDecoration(
          color: Theme.of(context).scaffoldBackgroundColor.darken(10),
          borderRadius: const BorderRadius.all(Radius.circular(20)),
        );
      }
    }
  }

  Widget _buildMenuItem(
    BuildContext context,
    RoutePath? routePath,
    IconData? iconData,
    String labelText,
    RoutePath? path,
  ) {
    final isSelected = routePath != null && path == routePath;

    // If routePath is null return menuItem without InkWell as google play store policy sees blank InkWell OnTap as Broken button on review.
    if (routePath == null) {
      return _menuItem(context, iconData, labelText, isSelected);
    }

    return InkWell(
      hoverColor: Colors.transparent,
      onTap: () {
        Provider.of<NavigationLogic>(context, listen: false)
            .setRoutePath(routePath);
      },
      child: _menuItem(context, iconData, labelText, isSelected),
    );
  }
}
